"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
const util = require("util");
const chalk_1 = require("chalk");
class LogFormatter extends Function {
    constructor() {
        super();
        // #region 私有属性与方法
        /**
         * 是否自动缩进对象输出
         */
        this._indentJson = false;
        /**
         * 使用console的何种方法打印到控制台
         */
        this._logType = 'log';
        /**
         * 样式层
         */
        this._formatLayer = [];
        // 第一层默认是时间
        this._formatLayer.push({
            style: chalk_1.default.gray,
            template: [v => `[${v}]`],
            get text() {
                switch (this.setting.timeFormat) {
                    case 0:
                        return moment().format('YYYY-MM-DD HH:mm:ss');
                    case 1: // noTime
                        return moment().format('YYYY-MM-DD');
                    case 2: // noDate
                        return moment().format('HH:mm:ss');
                }
            },
            setting: {
                hasUsed: true,
                timeFormat: 0,
                get skip() { return this.timeFormat > 2; }
            }
        });
        // 再添加一层是为了让用户可以在设置第一个样式之前可以不调用text
        this._unusedText;
    }
    /**
     * 添加一个未使用样式层
     */
    get _unusedText() {
        this._formatLayer.push({ style: chalk_1.default, template: [], setting: { hasUsed: false } });
        return this;
    }
    /**
     * 返回最后一个样式层
     */
    get _lastFormatLayer() {
        return this._formatLayer[this._formatLayer.length - 1];
    }
    /**
     * 获取最后一个样式层并将其设置为已经使用过了
     */
    get _getLastFormatLayerAndSetHasUsed() {
        const lastLayer = this._lastFormatLayer;
        lastLayer.setting.hasUsed = true;
        return lastLayer;
    }
    /**
     * 为最后一层设置样式，调用chalk的方法
     */
    _invokeChalkFunction(name, ...args) {
        const lastLayer = this._getLastFormatLayerAndSetHasUsed;
        lastLayer.style = lastLayer.style[name](...args);
        return this;
    }
    /**
     * 为最后一层设置样式，调用chalk的属性
     */
    _invokeChalkProperty(name) {
        const lastLayer = this._getLastFormatLayerAndSetHasUsed;
        lastLayer.style = lastLayer.style[name];
        return this;
    }
    // #endregion
    // #region 格式化输出
    /**
     * 格式化传入的参数，但不打印到console
     * 如果样式长度大于参数长度，则只应用对应部分，剩下的样式忽略。如果参数长度大于样式长度，则剩下的参数将应用最后一个样式
     */
    format(...args) {
        const result = [];
        const { style: lastLayerStyle, template: lastLayerTemplate } = this._lastFormatLayer;
        for (let argIndex = 0, formatIndex = 0; argIndex < args.length; formatIndex++) {
            if (formatIndex < this._formatLayer.length) {
                const { style, template, text, setting } = this._formatLayer[formatIndex];
                if (!setting.skip) { // 判断是否跳过当前层
                    if (text !== undefined)
                        var transformedString = template.reduce((pre, template) => template(pre), text);
                    else
                        var transformedString = template.reduce((pre, template) => template(pre), util.formatWithOptions({ compact: !this._indentJson }, args[argIndex++]));
                    result.push(style(transformedString));
                }
            }
            else {
                const transformedString = lastLayerTemplate.reduce((pre, template) => template(pre), util.formatWithOptions({ compact: !this._indentJson }, args[argIndex++]));
                result.push(lastLayerStyle(transformedString));
            }
        }
        return result.join(' ');
    }
    /**
     * 格式化传入的参数，并把结果打印到console
     */
    print(...args) {
        console[this._logType](this.format(...args));
    }
    /**
     * 打印一行分隔符。如果有多个样式被指定，则只有第一个会生效。时间不会被输出
     * @param char 分隔符字符
     * @param length 分隔符长度。默认等于终端窗口长度
     */
    line(char = '-', length = process.stdout.columns || 80) {
        const { style } = this._formatLayer[1];
        console[this._logType](style(char.repeat(length)));
    }
    // #endregion
    // #region 输出类型设置
    /**
     * 通过console.log进行输出，这个是默认选项
     */
    get log() {
        this._logType = 'log';
        return this;
    }
    /**
     * 通过console.error进行输出
     */
    get error() {
        this._logType = 'error';
        return this;
    }
    /**
     * 通过console.warn进行输出
     */
    get warn() {
        this._logType = 'warn';
        return this;
    }
    /**
     * 是否自动缩进对象输出
     */
    get indentJson() {
        this._indentJson = true;
        return this;
    }
    // #endregion
    // #region 时间格式设置
    /**
     * 不显示时间
     */
    get noTime() {
        this._formatLayer[0].setting.timeFormat |= 1;
        return this;
    }
    /**
     * 不显示日期
     */
    get noDate() {
        this._formatLayer[0].setting.timeFormat |= 2;
        return this;
    }
    /**
     * 不显示日期与时间
     */
    get noDatetime() {
        this._formatLayer[0].setting.timeFormat |= 3;
        return this;
    }
    // #endregion
    // #region 新建样式层属性
    /**
     * 创建一层新的样式，用于格式化下一个传入参数
     */
    get text() {
        const lastLayer = this._lastFormatLayer;
        if (lastLayer.setting.hasUsed)
            this._formatLayer.push({ style: chalk_1.default, template: [], setting: { hasUsed: true } });
        else
            lastLayer.setting.hasUsed = true; // 如果最后一层还没有用过就不在添加新的了
        return this;
    }
    /**
     * 消息的标题，text的别名
     */
    get title() {
        return this.text;
    }
    /**
     * 换行
     */
    get linefeed() {
        this.text._lastFormatLayer.text = '\r\n';
        return this._unusedText;
    }
    /**
     * 空格
     */
    get whitespace() {
        this.text._lastFormatLayer.text = ' ';
        return this._unusedText;
    }
    /**
     * 消息的正文，相当于linefeed.text
     */
    get content() {
        return this.linefeed.text;
    }
    /**
     * 代表消息发生的位置。相当于text.square
     */
    get location() {
        return this.text.square;
    }
    // #endregion
    // #region 样式模板
    /**
     * 用方括号包裹要输出的文本内容
     */
    get square() {
        this._getLastFormatLayerAndSetHasUsed.template.push(text => `[${text}]`);
        return this;
    }
    /**
     * 用圆括号包裹要输出的文本内容
     */
    get round() {
        this._getLastFormatLayerAndSetHasUsed.template.push(text => `(${text})`);
        return this;
    }
    /**
     * 用花括号包裹要输出的文本内容
     */
    get mustache() {
        this._getLastFormatLayerAndSetHasUsed.template.push(text => `{${text}}`);
        return this;
    }
    // #endregion
    // #region chalk的方法
    rgb(r, g, b) { return this._invokeChalkFunction('rgb', r, g, b); }
    hsl(h, s, l) { return this._invokeChalkFunction('hsl', h, s, l); }
    hsv(h, s, v) { return this._invokeChalkFunction('hsv', h, s, v); }
    hwb(h, w, b) { return this._invokeChalkFunction('hwb', h, w, b); }
    hex(color) { return this._invokeChalkFunction('hex', color); }
    keyword(color) { return this._invokeChalkFunction('keyword', color); }
    bgRgb(r, g, b) { return this._invokeChalkFunction('bgRgb', r, g, b); }
    bgHsl(h, s, l) { return this._invokeChalkFunction('bgHsl', h, s, l); }
    bgHsv(h, s, v) { return this._invokeChalkFunction('bgHsv', h, s, v); }
    bgHwb(h, w, b) { return this._invokeChalkFunction('bgHwb', h, w, b); }
    bgHex(color) { return this._invokeChalkFunction('bgHex', color); }
    bgKeyword(color) { return this._invokeChalkFunction('bgKeyword', color); }
    // #endregion
    // #region chalk的属性
    get reset() { return this._invokeChalkProperty('reset'); }
    get bold() { return this._invokeChalkProperty('bold'); }
    get dim() { return this._invokeChalkProperty('dim'); }
    get italic() { return this._invokeChalkProperty('italic'); }
    get underline() { return this._invokeChalkProperty('underline'); }
    get inverse() { return this._invokeChalkProperty('inverse'); }
    get hidden() { return this._invokeChalkProperty('hidden'); }
    get strikethrough() { return this._invokeChalkProperty('strikethrough'); }
    get visible() { return this._invokeChalkProperty('visible'); }
    get black() { return this._invokeChalkProperty('black'); }
    get red() { return this._invokeChalkProperty('red'); }
    get green() { return this._invokeChalkProperty('green'); }
    get yellow() { return this._invokeChalkProperty('yellow'); }
    get blue() { return this._invokeChalkProperty('blue'); }
    get magenta() { return this._invokeChalkProperty('magenta'); }
    get cyan() { return this._invokeChalkProperty('cyan'); }
    get white() { return this._invokeChalkProperty('white'); }
    get gray() { return this._invokeChalkProperty('gray'); }
    get grey() { return this._invokeChalkProperty('grey'); }
    get blackBright() { return this._invokeChalkProperty('blackBright'); }
    get redBright() { return this._invokeChalkProperty('redBright'); }
    get greenBright() { return this._invokeChalkProperty('greenBright'); }
    get yellowBright() { return this._invokeChalkProperty('yellowBright'); }
    get blueBright() { return this._invokeChalkProperty('blueBright'); }
    get magentaBright() { return this._invokeChalkProperty('magentaBright'); }
    get cyanBright() { return this._invokeChalkProperty('cyanBright'); }
    get whiteBright() { return this._invokeChalkProperty('whiteBright'); }
    get bgBlack() { return this._invokeChalkProperty('bgBlack'); }
    get bgRed() { return this._invokeChalkProperty('bgRed'); }
    get bgGreen() { return this._invokeChalkProperty('bgGreen'); }
    get bgYellow() { return this._invokeChalkProperty('bgYellow'); }
    get bgBlue() { return this._invokeChalkProperty('bgBlue'); }
    get bgMagenta() { return this._invokeChalkProperty('bgMagenta'); }
    get bgCyan() { return this._invokeChalkProperty('bgCyan'); }
    get bgWhite() { return this._invokeChalkProperty('bgWhite'); }
    get bgBlackBright() { return this._invokeChalkProperty('bgBlackBright'); }
    get bgRedBright() { return this._invokeChalkProperty('bgRedBright'); }
    get bgGreenBright() { return this._invokeChalkProperty('bgGreenBright'); }
    get bgYellowBright() { return this._invokeChalkProperty('bgYellowBright'); }
    get bgBlueBright() { return this._invokeChalkProperty('bgBlueBright'); }
    get bgMagentaBright() { return this._invokeChalkProperty('bgMagentaBright'); }
    get bgCyanBright() { return this._invokeChalkProperty('bgCyanBright'); }
    get bgWhiteBright() { return this._invokeChalkProperty('bgWhiteBright'); }
}
exports.LogFormatter = LogFormatter;
